/*============================================================================
  File:     DataTypeConsistency.sql

  Summary:  This script finds implicit conversions in the cache and 
            conversions that occur between columns being joined. 
			Then, it uses Jonathan's AWESOME DMW scripts to analyze
			cache to produce information about the query and the columns 
			that didn't match.
  
  SQL Server Versions: 2005/2008 and 2012
------------------------------------------------------------------------------
  Written by Jonathan M. Kehayias (all the XML stuff) 
    and the scenarios by Kimberly L. Tripp, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- These samples use the Credit database. You can download and restore the
-- credit database from here:
-- http://www.sqlskills.com/sql-server-resources/sql-server-demos/ 

-- NOTE: You can use a SQL Server 2000 back and restore to: 2000, 2005 or 2008/R2
-- There's also a 2008 backup that you can restore to 2008/R2 or 2012

USE [Credit];
GO

------------------------------------------------------
-- Implicit Conversions
------------------------------------------------------

-- This example will use LastNames to look up rows
-- An index can be helpful, especially when the data
-- is highly selective
CREATE INDEX [LastNameInd] 
ON [dbo].[member] ([lastname]);
GO

-- To ensure a highly selective value, let's set
-- one row to Tripp
UPDATE [dbo].[member]
    SET [lastname] = 'Tripp'
    WHERE [member_no] = 1234;
GO

-- To see the I/Os performed
SET STATISTICS IO ON;
GO

-- Turn on showplan (Query, Include Actual Execution Plan)

-- Execute these two statements TWICE 
-- (the plan won't be in cache if Opt for Adhoc Workloads is on)
SELECT [m].* 
FROM [dbo].[member] AS [m]
WHERE m.[lastname] = 'Tripp';
GO

SELECT [m].* 
FROM [dbo].[member] AS [m]
WHERE m.[lastname] = N'Tripp';
GO

-- Showplan percentages are different
--		SCAN vs. SEEK

-- Plan affects the I/Os and query cost

------------------------------------------------------
-- Checking the cache
------------------------------------------------------

-- For more information on ALL of the type conversions,
-- check out http://bit.ly/XFS6On 

-- Analyzing the plan cache can be incredibly useful
-- to find inconsistencies within your code
-- This code comes from Jon's blog post: 
--   Finding Implicit Column Conversions in the Plan Cache
-- http://bit.ly/17MdijL


SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED; 

DECLARE @dbname SYSNAME 
SET @dbname = QUOTENAME (DB_NAME ()); 

WITH XMLNAMESPACES 
   (DEFAULT 'http://schemas.microsoft.com/sqlserver/2004/07/showplan') 
SELECT 
   [stmt].[value](N'(@StatementText)[1]', N'VARCHAR(MAX)'), 
   [t].[value](N'(ScalarOperator/Identifier/ColumnReference/@Schema)[1]', N'VARCHAR(128)'), 
   [t].[value](N'(ScalarOperator/Identifier/ColumnReference/@Table)[1]', N'VARCHAR(128)'), 
   [t].[value](N'(ScalarOperator/Identifier/ColumnReference/@Column)[1]', N'VARCHAR(128)'), 
   [ic].[DATA_TYPE] AS [ConvertFrom], 
   [ic].[CHARACTER_MAXIMUM_LENGTH] AS [ConvertFromLength], 
   [t].[value](N'(@DataType)[1]', N'VARCHAR(128)') AS [ConvertTo], 
   [t].[value](N'(@Length)[1]', N'INT') AS [ConvertToLength], 
   [query_plan] 
FROM [sys].[dm_exec_cached_plans] AS [cp]
	CROSS APPLY [sys].[dm_exec_query_plan]([plan_handle]) AS [qp]
	CROSS APPLY [query_plan].[nodes](
		'/ShowPlanXML/BatchSequence/Batch/Statements/StmtSimple')
		AS [batch]([stmt]) 
	CROSS APPLY [stmt].[nodes](
		'.//Convert[@Implicit="1"]') AS [n]([t]) 
JOIN [INFORMATION_SCHEMA].[COLUMNS] AS [ic]
   ON QUOTENAME ([ic].[TABLE_SCHEMA]) 
		= [t].[value](N'(ScalarOperator/Identifier/ColumnReference/@Schema)[1]', N'VARCHAR(128)') 
   AND QUOTENAME ([ic].[TABLE_NAME]) 
		= [t].[value](N'(ScalarOperator/Identifier/ColumnReference/@Table)[1]', 'VARCHAR(128)') 
   AND [ic].[COLUMN_NAME] 
		= [t].[value]('(ScalarOperator/Identifier/ColumnReference/@Column)[1]', 'VARCHAR(128)') 
WHERE [t].[exist]('ScalarOperator/Identifier/ColumnReference[@Database=sql:variable("@dbname")][@Schema!="[sys]"]') 
		= 1 
OPTION (MAXDOP 1);
GO